#==================================================================================
# Orinal author: Ananda Mahto
# Original code: https://gist.github.com/mrdwab/7586769
# Adapted by   : Jos Cludio Faria
# Objective    : To supply the current necessity of the Tinn-R project
# Date         : 2023-05-19 23:21:33
#==================================================================================
TR_Args_Description <- function(object,
                                package=NULL,
                                section='Arguments',
                                sep='|',
                                path=NULL)
{
  object <- deparse(substitute(object))
  pkg_ref <- package  # See help(help) last example

  x <- capture.output(tools:::Rd2txt(utils:::.getHelpFile(utils::help(topic=object,
                                                                      (pkg_ref))),
                                     options=list(sectionIndent=0)))
  B <- grep('^_',
            x)                                ## section start lines

  x <- gsub('_\b',
            '',
            x,
            fixed=TRUE)                       ## remove '_\b'

  X <- rep(FALSE,
           length(x))                         ## Create a FALSE vector

  X[B] <- 1                                   ## Initialize

  res <- split(x,
               cumsum(X))                     ## Create a list of sections

  sectionID <- vapply(res,
                      function(x)             ## Identify where the section starts
    grepl(section, x[1],
          fixed=TRUE),
          logical(1L))

  if (!any(sectionID)) {                      ## If the section is missing...
    ''                                        ## ... just return an empty character
  } else {                                    ## Else, get that list item
    res <- res[[which(sectionID)]][-c(1, 2)]
    while(TRUE) {                             ## Remove the extra empty lines
      res <- res[-length(res)]                ## from the end of the file
      if (res[length(res)] != '')
        break
    }

    # The result
      if(is.null(path))
        return(res)
      else {
        sink(path)
         write.table(res,
                     row.names=FALSE,
                     col.names=FALSE,
                     sep=sep,
                     qmethod='double')
        sink()
      }
  }
}
