#=======================================================================
# Orinal author : Jos Cludio Faria
# Objective     : To supply the current necessity of the Tinn-R project
# Date          : 2023-04-24 15:08:47
#=======================================================================

TR_Code_Completion <- function(object,
                               slots=c('name',
                                       'title',
                                       'description'),
                               sep='|',
                               path=NULL)
{
  if(nchar(object) < 2)
    stop('Use at least three characters!')
  
  # Get all objects (named)
  res <- apropos(object,
                 where=TRUE,
                 ignore.case=FALSE)

  if(length(res) <= 0)
    stop(paste0('No objects found starting with: ',
                object))

  nam_res <- names(res)
  obj_res <- as.character(res)
  obj_packs <- search()[as.integer(nam_res)]

  obj_ok <- grepl('^package:',
                  obj_packs)
                
  obj_res <- obj_res[obj_ok]
  packs <- obj_packs[obj_ok]
  packs <- gsub(pattern='^package:',
                replacement='',
                x=packs)

  # Get documentation for all objects
  help_list <- list()
  for (i in 1:length(obj_res))
  {
    help_list[[paste(obj_res[i],
                     packs[i],
                     sep=':')]] <- try(TR_Get_Help(obj_res[i],
                                                   packs[i],
                                                   slots),
                                       silent=TRUE)
  }

  # Describe all objects
  obj_doc <- gsub(pattern='[:].+$',
                  replacement='',
                  x=names(help_list))

  # Make a data.frame of the results
  res <- data.frame()
  for (i in 1:length(help_list))
  {
    s1=s2=s3=0
    res[i,1] <- obj_doc[i]
    res[i,2] <- packs[i]
    if ('name' %in% slots)
      res[i,3] <- help_list[[i]]$name
    else
      res[i,3] <- NA
    if ('title' %in% slots)
      res[i,4] <- help_list[[i]]$title
    else
      res[i,4] <- NA
    if ('description' %in% slots)
      res[i,5] <- help_list[[i]]$description
    else
      res[i,5] <- NA
    res[i, 6] <- is.function(eval(as.name(obj_doc[i])))
  }

  # Remove columns with NA Values Using Base R
  maintain_cols <- (apply(is.na(res), 2, sum) == 0)
  res <- res[, maintain_cols]

  names(res) <- c('object',
                  'package',
                   slots,
                  'function')

# The result
  if(is.null(path))
    return(res)
  else {
    sink(path)
     cat(length(help_list),
         '\n')
     write.table(res,
                 row.names=FALSE,
                 col.names=FALSE,
                 sep=sep,
                 qmethod='double')
    sink()
  }
}
